// TODO merge buffered_reader and buffered_writer to buffered_io

/* ------- buffered_reader.c ---------------------------

  Read or write a file bytewise through a buffer for speed.

  ---------------------------------------------------- */

#include <stdio.h>
#include "buffered_io.h"

#define BUF_SIZE      128
#define BUF_MAX_INDEX 127

uint8_t fbuf[BUF_SIZE];
uint8_t fbufidx;
size_t recs;
FILE *fp;


// DEBUG >>>
/*
void dumpfbuf() {
    int i;
    int cnt;
    
    cnt = 0;
    for (i = 0; i < BUF_SIZE; i++) {
        printf("%2x ", fbuf[i]);
        cnt++;
        if (cnt > 0x0F) {
            cnt = 0;
            putchar('\n');
        }
    }
    printf("ENDE\n");
    getchar();
}
*/
// <<< DEBUG


// Opens a buffered file.
errorCode_t openBufferedFile(const char *filename, const char *mode) {
    if (*mode == 'r') {
        fbufidx = BUF_MAX_INDEX; // force block read at first readBufferedByte
    }
    else {
        fbufidx = 0; // write access: start with empty buffer
    }
    fp = fopen(filename, mode);
  
    return fp == NULL ? ErrCouldNotOpenFile : ErrOk;
}


/* Reads a byte from the buffer. Reads a block from disc
   as soon as the buffer is read to the end. */
uint8_t readBufferedByte() {
    if (fbufidx == BUF_MAX_INDEX) {
        fbufidx = 0;
        recs = fread(fbuf, BUF_SIZE, 1, fp); // reads a block of 128 bytes
        //dumpfbuf(); // DEBUG
    }
    else {
        ++fbufidx;
    }
  
    return fbuf[fbufidx];
}


/* Writes a byte to the buffer. Writes a block to disc
   as soon as the buffer is full. */
void writeBufferedByte(uint8_t b) {
    fbuf[fbufidx++] = b;

    if (fbufidx == BUF_SIZE) {
        fbufidx = 0;
        recs = fwrite(fbuf, BUF_SIZE, 1, fp); // writes a block of 128 bytes
        //dumpfbuf(); // DEBUG
    }
}


// Flushes the buffer
void flush(void) {
    if (fbufidx > 0) {
        recs = fwrite(fbuf, fbufidx, 1, fp); // writes the current buffer content
        fbufidx = 0;
    }
}


// End of buffered file reached?
uint8_t endOfBufferedFile() {
    return recs == 0;
}


// Closes the buffered file previously opened.
void closeBufferedFile() {
    fclose(fp);
}
