/* ------- buffered_reader.h ---------------------------

  Read or write a file bytewise through a buffer for speed.

  ---------------------------------------------------- */

#ifndef __BUFFERED_IO_H__
#define __BUFFERED_IO_H__

#include <stdint.h>

typedef enum {ErrOk, ErrCouldNotOpenFile} errorCode_t;


// Opens a buffered file.
errorCode_t openBufferedFile(const char *filename, const char *mode);

/* Reads a byte from the buffer. Reads a block from disc
   as soon as the buffer is read to the end. */
uint8_t readBufferedByte();

/* Writes a byte to the buffer. Writes a block to disc
   as soon as the buffer is full. */
void writeBufferedByte(uint8_t b);

// Flushes the buffer.
void flush(void);

// End of buffered file reached?
uint8_t endOfBufferedFile();

// Closes the buffered file previously opened.
void closeBufferedFile();

#endif
