/*
  Graphics commands for Schneider JOYCE.
  This is the main module, containing machine code and elementary
  routines.

     gx_init          -> Initializes the machine code
     gx_plot(x, y, m) -> Pixel set/reset/invert
     gx_screenOn      -> Switch on screen
     gx_screenOff     -> Switch off screen
     gx_showCursor    -> Cursor on
     gx_hideCursor    -> Cursor off
     gx_dotColor      -> retrieve pixel color

   Variables and procedures use a gx_ prefix to avoid name collisions
   with other modules.

   When compiling, make sure the machine code does not get overwritten.
   Use the zcc option -pragma-define:REGISTER_SP=0xF3FF.
*/

#include <conio.h>
#include <string.h>

#define XBIOS   0xFC5A
#define RUN_SCR 0x00E9

void *gx_dot(void)    @ 0xF400;
void *gx_symb(void)   @ 0xF462;
void *gx_get(void)    @ 0xF47F;
void *gx_scrAcc(void) @ 0xF494;

unsigned int  gx_xMC       @ 0xF410;
unsigned char gx_yMC       @ 0xF412;
unsigned char gx_modeMC    @ 0xF413; // set=0 / reset=1 / invert=2
unsigned char gx_rowMC     @ 0xF4A0;
unsigned char gx_colMC     @ 0xF49E;
unsigned char gx_accModeMC @ 0xF4B9;

/* Initializes the machine code.
 */
void gx_init(void) {
    unsigned char *p;
    unsigned char c;
    unsigned char mCode[203] =
      {0xED,0x5B,0x10,0xF4,0x2A,0x12,0xF4,0x01,0x14,0xF4,0xCD,0x5A,0xFC,0xE9,0x00,0xC9,
       0x00,0x00,0x00,0x00,0x3E,0x03,0xA2,0x57,0xE5,0x26,0x00,0x29,0x01,0x00,0xB6,0x09,
       0x4E,0x23,0x46,0x79,0xE6,0xF8,0x6F,0x60,0x29,0x19,0x7D,0xE6,0xF8,0x6F,0x79,0xE6,
       0x07,0xB5,0x6F,0x7B,0xE6,0x07,0x3C,0x47,0xAF,0x37,0x1F,0x10,0xFD,0xC1,0x4F,0x78,
       0xFE,0x00,0x20,0x04,0x79,0xB6,0x77,0xC9,0xFE,0x01,0x20,0x05,0x79,0x2F,0xA6,0x77,
       0xC9,0xFE,0x02,0x20,0x04,0x79,0xAE,0x77,0xC9,0xFE,0x03,0xC0,0x79,0xA6,0x32,0x13,
       0xF4,0xC9,0x01,0x6B,0xF4,0xCD,0x5A,0xFC,0xE9,0x00,0xC9,0x21,0x77,0xF4,0x01,0x08,
       0x00,0x11,0x00,0x00,0xED,0xB0,0xC9,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,
       0x88,0xF4,0xCD,0x5A,0xFC,0xE9,0x00,0xC9,0x21,0x00,0x00,0x01,0x08,0x00,0x11,0x77,
       0xF4,0xED,0xB0,0xC9,0x01,0x9D,0xF4,0xCD,0x5A,0xFC,0xE9,0x00,0xC9,0x1E,0x00,0x3E,
       0x00,0x16,0x00,0x07,0x17,0x17,0x17,0x6F,0x7A,0xCE,0xB6,0x67,0x7E,0x23,0x66,0x6F,
       0xEB,0x29,0x29,0x19,0x29,0x01,0x08,0x00,0x3E,0x00,0xFE,0x00,0x20,0x06,0x11,0x77,
       0xF4,0xED,0xB0,0xC9,0xEB,0x21,0x77,0xF4,0xED,0xB0,0xC9};

    memcpy(0xF400, mCode, 203);
}

/* Plots a dot.
   Parameters: x     X coordinate
               y     Y coordinate
               mode  set=0 / reset=1 / invert=2
 */
void gx_plot(unsigned int x, unsigned char y, unsigned char mode) {
    gx_xMC = x;
    gx_yMC = y;
    gx_modeMC = mode;
    gx_dot();
}

/* Gets the color of a dot.
   Parameters: x     X coordinate
               y     Y coordinate
   Returns 1 if set, 0 else
 */
unsigned char gx_dotColor (unsigned int x, unsigned char y) {
  gx_plot(x, y, 3);
  return *((unsigned char *) 0xF413) != 0;
}

/* TODO >>>
void gx_screenOn(void) {
}
PROCEDURE Screen_On;
begin
  port(.248.):=7;
end;

void gx_screenOff(void) {
}
PROCEDURE Screen_Off;
begin
  port(.248.):=8;
end;
<<< TODO */

/* Hides the text cursor.
 */
void gx_hideCursor(void) {
    putch(27);
    putch('f');
}

/* Shows the text cursor.
 */
void gx_showCursor(void) {
    putch(27);
    putch('e');
}
