#ifndef __KERNEL_H__
#define __KERNEL_H__

/*
  Graphics commands for Schneider JOYCE.
  This is the main module, containing machine code and elementary
  routines.

     gx_init          -> Initializes the machine code
     gx_plot(x, y, m) -> Pixel set/reset/invert
     gx_screenOn      -> Switch on screen
     gx_screenOff     -> Switch off screen
     gx_showCursor    -> Cursor on
     gx_hideCursor    -> Cursor off
     gx_dotColor      -> retrieve pixel color

   Variables and procedures use a gx_ prefix to avoid name collisions
   with other modules.

   When compiling, make sure the machine code does not get overwritten.
   Use the zcc option -pragma-define:REGISTER_SP=0xF3FF.
*/

extern void *gx_dot(void);
extern void *gx_symb(void);
extern void *gx_get(void);
extern void *gx_scrAcc(void);

extern unsigned int  gx_xMC;
extern unsigned char gx_yMC;
extern unsigned char gx_modeMC; // set=0 / reset=1 / invert=2
extern unsigned char gx_rowMC;
extern unsigned char gx_colMC;
extern unsigned char gx_accModeMC;

/* Initializes the machine code.
 */
void gx_init(void);

/* Plots a dot.
   Parameters: x     X coordinate
               y     Y coordinate
               mode  set=0 / reset=1 / invert=2
 */
void gx_plot(unsigned int x, unsigned char y, unsigned char mode);

/* Gets the color of a dot.
   Parameters: x     X coordinate
               y     Y coordinate
   Returns 1 if set, 0 else
 */
unsigned char gx_dotColor (unsigned int x, unsigned char y);

/* TODO >>>
void gx_screenOn(void);

void gx_screenOff(void);
<<< TODO */

/* Hides the text cursor.
 */
void gx_hideCursor(void);

/* Shows the text cursor.
 */
void gx_showCursor(void);

#endif /* __KERNEL_H__ */
