/*
  Save a screen dump to a PBM (portable bitmap) file.

    saveScreenToPBM -> Get image row from the Roller RAM
                       and save it to a PBM file
*/

#include "kernel.h"
#include "screen.h"
#include "buffered_io.h"
#include "mytypes.h"

#define BUF_SIZE 128


/* Writes a line of pixels.
 */
void writePixelLine(unsigned char y) {
    unsigned char x;
    unsigned int i;

    i = y;
    for (x = 0; x < 90; x++) { // 90 = 720 / 8
        writeBufferedByte(gx_buffer[i]);
        i += 8;
    }
}

/* Saves a screen dump to a black and white 1 bit per pixel PBM file.
 */
void saveScreenToPBM(const char *filename, bool doubleHeight) {
    unsigned char row;
    unsigned char x, y;
    unsigned int  i;
    errorCode_t errCode;

    errCode = openBufferedFile(filename, "wb");
    if (errCode == ErrOk) {
        // Write header
        writeBufferedByte(0x50);     // 'P'
        writeBufferedByte(0x34);     // '4'
        writeBufferedByte(0x0A);     // LF
        writeBufferedByte(0x37);     // '7'
        writeBufferedByte(0x32);     // '2'
        writeBufferedByte(0x30);     // '0'
        writeBufferedByte(0x20);     // ' '
        if (doubleHeight) {
            writeBufferedByte(0x35); // '5'
            writeBufferedByte(0x31); // '1'
            writeBufferedByte(0x32); // '2'
        }
        else {
            writeBufferedByte(0x32); // '2'
            writeBufferedByte(0x35); // '5'
            writeBufferedByte(0x36); // '6'
        }
        writeBufferedByte(0x0A);     // LF 

        // Write pixel data
        for (row = 0; row < 32; row++) {
            gx_getScreenRow(row); // gets 720 bytes = 8 rows of 90 bytes
            for (y = 0; y < 8; y++) {
                writePixelLine(y);
                if (doubleHeight) {
                    writePixelLine(y);
                }
            }
        }
        
        flush();
        closeBufferedFile();
    }
}
