#include <stdio.h>
#include <string.h>
#include "readscene.h"

#define MAX_TOKEN_LENGTH 20

float toFloat(char *s);
unsigned char isWhitespace(char ch);

// Global variables
t_scene scene;


/* Initializes the scene.
 */
void initScene(t_scene *scene) {
    int i;
    t_coords3d sphereCoords;

    scene->camera.x = 0.0f;
    scene->camera.y = 0.0f;
    scene->camera.z = 0.0f;

    scene->sphereCount = 0;

    for (i = 0; i < scene->sphereCount; ++i) {
        sphereCoords = scene->sphere[i];
        sphereCoords.x = 0.0f;
        sphereCoords.y = 0.0f;
        sphereCoords.z = 0.0f;

        scene->radius[i] = 0.0f;
    }
}

/* Prints the scene to stdout. For testing purposes.
 *
void printScene(t_scene *scene) {
    t_coords3d coords;
    int i;

    coords = scene->camera;
    printf("camera: x=%f y=%f z=%f\n", coords.x, coords.y, coords.z);
    printf("sphereCount: %d\n", scene->sphereCount);
    for (i = 0; i < scene->sphereCount; ++i) {
        coords = scene->sphere[i];
        printf("sphere %d: x=%f y=%f z=%f\n", i, coords.x, coords.y, coords.z);
    }
    printf("radius:");
    for (i = 0; i < scene->sphereCount; ++i) {
        printf(" %f", scene->radius[i]);
    }
    printf("\n");
}
*/

/* Parses a token from the line.
 */
char* parseToken(char *start, char *field) {
	unsigned char len;
	
	// skip whitespace
	while (isWhitespace(*start)) {
		++start;
	}
	
	// copy payload
	len = 0;
    while (!isWhitespace(*start) && *start != '\0') {
        *field++ = *start++;
		if (++len == MAX_TOKEN_LENGTH) {
			break;
		}
    }
    *field = '\0'; // end string

    return ++start;
}

/* Parses the line in the buffer.
 */
void parseLine(char *buf, t_scene *scene) {
	unsigned char sphereIdx;
    int i;
    int len;
	char token[MAX_TOKEN_LENGTH + 1];

    buf = parseToken(buf, token); // keyword
	if (strcmp("camera", token) == 0) {
		buf = parseToken(buf, token); // x
		scene->camera.x = toFloat(token);
		buf = parseToken(buf, token); // y
		scene->camera.y = toFloat(token);
		buf = parseToken(buf, token); // z
		scene->camera.z = toFloat(token);
	}
	else if (strcmp("sphere", token) == 0) {
		sphereIdx = scene->sphereCount;

		if (sphereIdx < MAX_SPHERES) {
			buf = parseToken(buf, token); // x
			scene->sphere[sphereIdx].x = toFloat(token);
			buf = parseToken(buf, token); // y
			scene->sphere[sphereIdx].y = toFloat(token);
			buf = parseToken(buf, token); // z
			scene->sphere[sphereIdx].z = toFloat(token);
			buf = parseToken(buf, token); // radius
			scene->radius[sphereIdx] = toFloat(token);
			
			scene->sphereCount = ++sphereIdx;
		}
	}
}

/* Reads the raytracing scene.
 */
t_scene *readScene(const char *filename) {
	FILE* fp;
    char buf[256];

    initScene(&scene);

	fp = fopen(filename, "r");
	if (fp == NULL) {
		fprintf(stderr, "Error: could not open config file %s!\n", filename);
		return -1;
	}

    // read line by line
    while (fgets(buf, sizeof buf, fp) != NULL) {
		if (*buf != '#') { // ignore comment lines
			parseLine(buf, &scene);
		}
    }

	fclose(fp);

    return &scene;
}

/* Converts a string to a float value.
 */
float toFloat(char *s) {
	float result;
	char decimalPointFound;
	char negative;
	char digit;
	float factor;
	
	result = 0.0f;
	decimalPointFound = 0;
	negative = 0;
	factor = 10.0f;
	
	while (*s != '\0') {
		if (*s >= '0' && *s <= '9') {
			digit = *s - '0';
			if (decimalPointFound) {
				result = result + (float)digit / factor;
				factor *= 10.0f;
			}
			else {
				result = 10.0f * result + digit;
			}
		}
		else if (*s == '.') {
			decimalPointFound = 1;
		}
		else if (*s == '-') {
			negative = 1;
		}
		
		++s;
	}
	
	if (negative) {
		result = -result;
	}
	
	return result;
}

/* Checks if a character is a whitespace.
 */
unsigned char isWhitespace(char ch) {
	return ch == ' ' || ch == '\t' || ch == '\n';
}
